<?php

/**
 * WebTV
 *
 * Simple web interface to start/stop/set channel on a streaming
 * server fitted with a tv card
 *
 * @author Roland Eckert <roland.eckert@gmail.com>
 * @copyright 2006
 * @package webtv
 * @version 0.2
 */

class ObjectHandle {

  var $sObjectFile = '';

  function ObjectHandle() {
    $this->sObjectFile = constant('OBJECT_FILE');
  }

  function vSaveObject(&$poObj){
    if (!$fp=fopen($this->sObjectFile, 'w')){
      trigger_error('Error opening object file', E_USER_ERROR);
    }
    if(!fwrite($fp, serialize($poObj))){
      trigger_error('Error writing data to object file', E_USER_ERROR);
    }
    fclose($fp);
  }

  function vReadObject(){
    $_oReturn = false;
    if (is_file($this->sObjectFile)) {
      if(!$_oReturn=unserialize(file_get_contents($this->sObjectFile))){
        trigger_error('Error fetching object from file', E_USER_ERROR);
      }
    }
    return $_oReturn;
  }

}

class WebTv {

  var $aChannels = array();
  var $aStatus = array();

  var $sConfigXawtv = '';
  var $sShellCommand = '';

  function vInit() {
    $this->_vLog('initialising class webtv');

    $this->sConfigXawtv = constant('CONFIG_XAWTV');
    if (!is_file($this->sConfigXawtv)) {
      $this->_vLog('File not readable: '.$this->sConfigXawtv, MSG_ERROR);
      die();
    }

    $this->sShellCommand = 'sudo -u '.constant('STREAM_USER').' -H '.constant('SHELL_SCRIPT');
    if (!is_file(constant('SHELL_SCRIPT'))) {
      $this->_vLog('File not readable: '.constant('SHELL_SCRIPT'), MSG_ERROR);
      die();
    }

    $this->_vReadChannels();
		$this->_vParseRequest();
    $this->_vGetStatus();
    $this->_vShowHeader();
    $this->_vShowForm();
    $this->_vShowFooter();
  }

  function vPowerOn() {
    $this->_vExec('start', "");
	}

  function vPowerOff() {
    $this->_vExec('stop', "");
	}

  function vSetChannel($psChannel) {
    $this->_vExec('channel', $psChannel);
    $this->aStatus['channel'] = array('name'=>$psChannel,
                                      'number'=>$this->aChannels[$psChannel]);
  }

  function _vLog($psMessage, $pnLevel=MSG_INFO) {
    if (constant('DEBUG')===true) {
      switch ($pnLevel) {
        case MSG_ERROR:
          echo "<span style=\"color:red;\">$psMessage</span><br />\n";
        break;
        default:
          echo $psMessage."<br />\n";
        break;
      }
    }
  }

  function _vExec($psCommand, $psOptions='') {
    $_sCommand = escapeshellcmd($psCommand);
    $_sOptions = escapeshellarg($psOptions);
		$_sCommandLine = $this->sShellCommand.' '.$_sCommand.' '.$_sOptions;
    $this->_vLog('executing :'.$_sCommandLine);
    exec($_sCommandLine, $_aReturn, $_nReturn);
    $this->aStatus[$_sCommand] = array('code'=>(($_nReturn)?$_nReturn:0),
                                       'message'=>implode($_aReturn));
  }

  function _vGetStatus() {
		$this->_vExec('status');
	}

  function _vParseRequest(){

    if (isset($_REQUEST['sAction'])) {
      $this->sAction = $_REQUEST['sAction'];
    } else {
      $this->sAction = 'default';
    }
    $this->_vLog('parsing request: '.$this->sAction);

    switch($this->sAction) {
      case 'channel_set' :
        if (isset($_POST['sChannel'])) {
          if ($this->aStatus['status']['code']==1) {
            $this->vPowerOff();
						sleep(2);
          }
          $this->vSetChannel($_POST['sChannel']);
          $this->aStatus['lastmessage'] = 'channel set to '.$_POST['sChannel'];
          if ($this->aStatus['status']['code']==1) {
            $this->vPowerOn();
          }
        }
      break;
      default:
      case 'power_set' :
        if ($this->aStatus['status']['code']==1) {
          $this->vPowerOff();
          $this->aStatus['lastmessage'] = 'power off';
        } else {
          $this->vPowerOn();
          $this->aStatus['lastmessage'] = 'power on';
 				}
      break;
      default:
        $this->aStatus['lastmessage'] = '--';
      break;
    }

  }

  function _vReadChannels() {
    $this->aChannels = array();
    $this->_vLog('parsing xawtv config file: '.$this->sConfigXawtv);
    $aFile = parse_ini_file($this->sConfigXawtv, true);
		foreach ($aFile as $sKey=>$aEntry) {
      if (isset($aEntry['channel'])) {
        $this->aChannels[$sKey] = $aEntry['channel'];
      }
    }
    ksort($this->aChannels);
    $this->_vLog('number of channels found: '.count($this->aChannels));
  }

  function _vShowHeader() {
    $sHtmlOut = <<<EOD
<!DOCTYPE html
    PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" lang="en" xml:lang="en">
<head>
  <title>WebTv</title>
<style type="text/css">
body { font: normal 70%/1.3em sans-serif; padding: 1em; margin: 0; }
#divStatus {
  display: block;
  width: 400px;
  padding: .5em 0 .5em .5em;
  text-decoration: none;
  color: black;
  border: 1px solid black;
  margin-bottom: 1px;
}
#divChannel,#divPower {
  display: block;
  width: 400px;
  padding: .5em 0 .5em .5em;
  text-decoration: none;
  color: black;
  border: 1px solid black;
  margin-bottom: 1px;
}
#divMessage {
  display: block;
  width: 400px;
  padding: .5em 0 .5em .5em;
  text-decoration: none;
  color: black;
  border: 1px solid black;
  margin-bottom: 1px;
}
</style>
</head>
<body>
EOD;
    echo $sHtmlOut;
  }

  function _vShowForm(){
    $sChannel = '<select name="sChannel">';
    foreach ($this->aChannels as $sKey=>$sValue) {
      $sSelected = ($sKey==$this->aStatus['channel']['name']) ? ' selected="selected"' : '';
      $sChannel.= '<option'.$sSelected.'>'.$sKey.'</option>';
    }
    $sChannel.= '</select>';
    $sHtmlOut = <<<EOD
<div id="divStatus">
  Status: {$this->aStatus['status']['message']}<br />
  Last Channel: {$this->aStatus['channel']['name']} ({$this->aStatus['channel']['number']})<br />
</div>
<div id="divChannel">
  <form name="frmChannel" method="post" action="{$_SERVER['REQUEST_URI']}">
    <input type="hidden" name="sAction" value="channel_set">
    <label for="nChannel">Channel</label> $sChannel
    <input type="submit" value="Set channel" name="btnSubmit" />
  </form>
</div>
<div id="divPower">
  <form name="frmPower" method="post" action="{$_SERVER['REQUEST_URI']}">
    <input type="hidden" name="sAction" value="power_set">
    <input type="submit" value="Power On/Off" name="btnSubmit" />
  </form>
</div>
<div id="divMessage">
  Last Message: {$this->aStatus['lastmessage']}<br />
</div>
EOD;
    echo $sHtmlOut;
  }

  function _vShowFooter() {
    $sHtmlOut = <<<EOD
 </body>
</html>
EOD;
    echo $sHtmlOut;
  }


}

/**
 * debug messages/error reporting settings
 */
define('DEBUG', false);
if (constant('DEBUG')===true) {
  error_reporting(E_ALL|E_STRICT);
  ini_set('display_errors','on');
} else {
  error_reporting(E_NONE);
  ini_set('display_errors','off');
}

/**
 * constant and path declarations
 */
define('OBJECT_FILE','/tmp/webtv.object');
define('STREAM_USER', 'stream');
define('SHELL_SCRIPT', '/home/stream/stream.sh');
define('CONFIG_XAWTV', '/home/stream/.xawtv');

/**
 * debug message levels
 */
define('MSG_INFO', 1);
define('MSG_ERROR', 5);


/**
 * try to read object from file
 */
$oOH = new ObjectHandle();
$oWebTv = $oOH->vReadObject();
if(!$oWebTv) {
  $oWebTv = new WebTv();
}
$oWebTv->vInit();
$oOH->vSaveObject($oWebTv);
?>